from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import commands
import os
import time
import Queue


class SysInfoReader(Sensor):

    # the maximum number of fields
    SIZE = 5

    def __init__(self):

        global _; _ = i18n.Translator("sysinfo-sensor")

        self.__cpu_run   = False

        self.__fonts = ""

        self.__key_color = ""
        self.__val_color = ""

        self.__current_size = 0
        self.__size_changed = 0

        self.__path  = os.path.join(os.getcwd(), "gfx")
        self.__queue = Queue.Queue(1)

        fail, self.__arch = commands.getstatusoutput("uname -m")

        Sensor.__init__(self)

        self._set_config_type("update", TYPE_INT, 1000)
        self._set_config_type("fonts", TYPE_STRING, "Sans 10")
        self._set_config_type("key_color", TYPE_STRING, "white")
        self._set_config_type("val_color", TYPE_STRING, "yellow")
        self._set_config_type("show_cpu", TYPE_BOOL, 1)
        self._set_config_type("show_hdd", TYPE_BOOL, 1)
        self._set_config_type("show_ram", TYPE_BOOL, 1)
        self._set_config_type("show_net", TYPE_BOOL, 1)

        self._add_thread(self.__get_sys_info_thread)
        self._add_timer(100, self.__get_sys_info)



    def get_configurator(self):

        configurator = self._new_configurator()
        configurator.set_name(_("SysInfo"))
        configurator.add_title(_("Settings"))
        configurator.add_checkbox(_("Show CPU Information"), "show_cpu",
                                  _("If enabled, shows the CPU display"))
        configurator.add_checkbox(_("Show Harddisk Information"), "show_hdd",
                                  _("If enabled, shows the HDD display"))
        configurator.add_checkbox(_("Show RAM Information"), "show_ram",
                                  _("If enabled, shows the RAM display"))
        configurator.add_checkbox(_("Show Network Traffic"), "show_net",
                                  _("If enabled, shows the network traffic "
                                    "display"))
        configurator.add_title(_("WARNING:\nIf you set the update rate to a "
                                 "LOW value,\nit will result in a HIGH load!"))
        configurator.add_spin(_("Update every (ms):"), "update",
                              _("Updates the display(s) every x milliseconds"),
                              1000, 10000)
        configurator.add_font_selector(_("Font:"), "fonts", _("The font which "
                                       "is used to display the labels"))
        configurator.add_color_selector(_("Key color:"), "key_color",
                                        _("The color of the key"))
        configurator.add_color_selector(_("Value color:"), "val_color",
                                        _("The color of the value"))

        return configurator


    def __get_cpu_data(self):

        fd = open("/proc/cpuinfo", "r")
        data = fd.read()
        fd.close()
        lines = data.splitlines()

        if (self.__arch in ["i386", "i486", "i586", "i686"]):

            if (not self.__cpu_run):
                # x86
                for l in lines[4:7]:
                    l = l.strip()
                    if (l.startswith("model")):
                        value = l.replace(":", " ")
                        value = value.split()
                        cpu_type = value[2] + " " + value[3]
                    if (l.startswith("cpu")):
                        value = l.replace(":", " ")
                        value = value.split()
                        cpu_speed = value[2] + " " + value[1]
 
                        cpu_temp = "???"

        elif (self.__arch == "ppc"):
            # powerpc
            for l in lines[0:3]:
                l = l.strip()
                if (l.startswith("cpu")):
                    value = l.replace(":", " ")
                    value = value.split()
                    cpu_type = value[1]
                if (l.startswith("clock")):
                    value = l.replace(":", " ")
                    value = value.split()
                    cpu_speed = value[1]
                if (l.startswith("temperature")):
                    value = l.replace(":", " ")
                    value = value.split()
                    cpu_temp = value[1] + " " + value[2]

        else:
            # TODO (sparc/alpha/mips)
            pass

        self.__run_cpu = True

        return (cpu_type, cpu_speed, cpu_temp)



    def __do_cpu(self, signal, index):

        cpu_type, cpu_speed, cpu_temp = self.__get_cpu_data()

        signal.set("icon[%(index)d]"   % vars(), os.path.join(self.__path,
                                                              "cpu.png"))
        signal.set("key1[%(index)d]"   % vars(), _("Type:"))
        signal.set("value1[%(index)d]" % vars(), cpu_type)
        signal.set("key2[%(index)d]"   % vars(), _("Clock:"))
        signal.set("value2[%(index)d]" % vars(), cpu_speed)
        signal.set("key3[%(index)d]"   % vars(), _("Temp:"))
        signal.set("value3[%(index)d]" % vars(), cpu_temp)
        signal.set("show1[%(index)d]"  % vars(), 1)
        signal.set("show2[%(index)d]"  % vars(), 1)
        signal.set("show3[%(index)d]"  % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 0)



    def __get_hdd_data(self):

        fail, hdd = commands.getstatusoutput("df")
        lines = hdd.splitlines()

        if (os.path.exists("/dev/.devfsd")):
            for l in lines[2:]:
                l = l.strip()
                value = l.split()
                hdd_total = int(value[0]) / 1024
                hdd_used  = int(value[1]) / 1024
                hdd_free  = int(value[2]) / 1024
                #hdd_perc  = int((float(hdd_used) / float(hdd_total)) * 100)
                hdd_total = str(hdd_total) + " MB"
                hdd_used  = str(hdd_used) + " MB"
                hdd_free  = str(hdd_free) + " MB"

        else:
            for l in lines[1:]:
                l = l.strip()
                value = l.split()
                hdd_total = int(value[1]) / 1024
                hdd_used  = int(value[2]) / 1024
                hdd_free  = int(value[3]) / 1024
                #hdd_perc  = int((float(hdd_used) / float(hdd_total)) * 100)
                hdd_total = str(hdd_total) + " MB"
                hdd_used  = str(hdd_used) + " MB"
                hdd_free  = str(hdd_free) + " MB"

        return (hdd_total, hdd_used, hdd_free)



    def __do_hdd(self, signal, index):

        hdd_total, hdd_used, hdd_free = self.__get_hdd_data()

        signal.set("icon[%(index)d]"      % vars(), os.path.join(self.__path,
                                                                 "hdd.png"))
        signal.set("key1[%(index)d]"      % vars(), _("Total:"))
        signal.set("value1[%(index)d]"    % vars(), hdd_total)
        signal.set("key2[%(index)d]"      % vars(), _("Used:"))
        signal.set("value2[%(index)d]"    % vars(), hdd_used)
        signal.set("key3[%(index)d]"      % vars(), _("Free:"))
        signal.set("value3[%(index)d]"    % vars(), hdd_free)
        #signal.set("fill[%(index)d]"      % vars(), hdd_perc)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        #signal.set("showgauge[%(index)d]" % vars(), 1)



    def __get_ram_data(self):        

        #fail, mem = commands.getstatusoutput("free")
        fd = open("/proc/meminfo", "r")
        mem = fd.read()
        fd.close()
        lines = mem.splitlines()

        for l in lines[1:2]:
            l = l.strip()
            if (l.startswith("Mem:")):
                value = l.split()
                total = float(value[1]) / (1024**2)
                used  = (float(value[3]) + float(value[6])) / (1024**2)
                free  = total - used
            if (l.startswith("Swap:")):
                pass
                
        #perc  = int((float(used) / float(total)) * 100)
        total = "%.02f" % total + " MB"
        used  = "%.02f" % used  + " MB"
        free  = "%.02f" % free  + " MB"

        return (total, used, free)



    def __do_ram(self, signal, index):

        total, used, free = self.__get_ram_data()

        signal.set("icon[%(index)d]"   % vars(), os.path.join(self.__path,
                                                              "memory.png"))
        signal.set("key1[%(index)d]"   % vars(), _("Total:"))
        signal.set("value1[%(index)d]" % vars(), total)
        signal.set("key2[%(index)d]"   % vars(), _("Used:"))
        signal.set("value2[%(index)d]" % vars(), used)
        signal.set("key3[%(index)d]"   % vars(), _("Free:"))
        signal.set("value3[%(index)d]" % vars(), free)   
        #signal.set("fill[%(index)d]"      % vars(), perc)
        signal.set("show1[%(index)d]"  % vars(), 1)
        signal.set("show2[%(index)d]"  % vars(), 1)
        signal.set("show3[%(index)d]"  % vars(), 1)
        #signal.set("showgauge[%(index)d]" % vars(), 1)



    def __get_sys_data(self):

        fail, uptime = commands.getstatusoutput("uptime")
        lines = uptime.splitlines()
        for l in lines:
            l = l.strip()
            l = l.replace(",", " ")
            value = l.split()
            if (value[3].startswith("day")):
                if (value[5] == "min"):
                    avg = value[12]
                    up = value[2] + " " + value[3] + " " + value[4] + " min"
                    users = value[6]
                else:
                    avg = value[11]
                    up = value[2] + " " + value[3] + " " + value[4] + " h"
                    users = value[5]
            else:
                if (value[3] == "min"):
                    avg = value[10]
                    up = value[2] + " minutes"
                    users = value[4]
                else:
                    avg = value[9]
                    up = value[2] + " hours"
                    users = value[3]

        return (avg, up, users)



    def __do_sys(self, signal, index):

        avg, up, users = self.__get_sys_data()

        signal.set("icon[%(index)d]"      % vars(), os.path.join(self.__path,
                                                                 "boot.png"))
        signal.set("key1[%(index)d]"      % vars(), _("Avg. Load:"))
        signal.set("value1[%(index)d]"    % vars(), avg)
        signal.set("key2[%(index)d]"      % vars(), _("Uptime:"))
        signal.set("value2[%(index)d]"    % vars(), up)
        signal.set("key3[%(index)d]"      % vars(), _("Users:"))
        signal.set("value3[%(index)d]"    % vars(), users)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 0)



    def __get_net_data(self):

        fd = open("/proc/net/dev", "r")
        data = fd.read()
        fd.close()
        lines = data.splitlines()
        for l in lines[2:]:
            l = l.strip()
            # Should we really read lo ?
            if (l.startswith("lo:")): continue
            if (l.startswith("eth")):
                value = l.replace(":", " ")
                value = value.split()
                ethX  = value[0]
                pack_in  = str("%.02f" % (float(value[1]) / (1024**2)) + " MB")
                pack_out = str("%.02f" % (float(value[9]) / (1024**2)) + " MB")

        return (ethX, pack_in, pack_out)



    def __do_net(self, signal, index):

        ethX, pack_in, pack_out = self.__get_net_data()

        signal.set("icon[%(index)d]"      % vars(),
                   os.path.join(self.__path, "network.png"))
        signal.set("key1[%(index)d]"      % vars(), _("Device:"))
        signal.set("value1[%(index)d]"    % vars(), ethX)
        signal.set("key2[%(index)d]"      % vars(), _("Bytes in:"))
        signal.set("value2[%(index)d]"    % vars(), pack_in)
        signal.set("key3[%(index)d]"      % vars(), _("Bytes out:"))
        signal.set("value3[%(index)d]"    % vars(), pack_out)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 0)



    def __update_config(self):

        update    = self._get_config("update")
        fonts     = self._get_config("fonts")
        key_color = self._get_config("key_color")
        val_color = self._get_config("val_color")

        v1 = (self.__fonts, self.__key_color, self.__val_color)
        v2 = (fonts, key_color, val_color)
        
        # only take action when some value has changed
        if (v1 != v2 or self.__size_changed):
            signal = self._new_output()

            if (self.__fonts != fonts or self.__size_changed):
                for i in xrange(self.SIZE):
                    signal.set("fonts[%(i)d]" %vars(), fonts)
            if (self.__key_color != key_color or self.__size_changed):
                for i in xrange(self.SIZE):
                    signal.set("key_color[%(i)d]" %vars(), key_color)
            if (self.__val_color != val_color or self.__size_changed):
                for i in xrange(self.SIZE):
                    signal.set("val_color[%(i)d]" %vars(), val_color)

            self._send_output(signal)

            self.__fonts        = fonts
            self.__key_color    = key_color
            self.__val_color    = val_color
            self.__size_changed = 0



    def __get_sys_info(self):

        self.__update_config()
        if (not self.__queue.empty()):
            signal = self.__queue.get()
            self._send_output(signal)

            size = signal.get("size")
            if (size != self.__current_size):
                self.__current_size = size
                self.__size_changed = 1

        return 1



    def __get_sys_info_thread(self):

        while(1):
            if (self._is_stopped()): break
            index     = 0
            update    = self._get_config("update")

            signal = self._new_output()

            self.__do_sys(signal, index)
            index += 1

            if (self._get_config("show_cpu")):
                self.__do_cpu(signal, index)
                index += 1
            if (self._get_config("show_hdd")):
                self.__do_hdd(signal, index)
                index += 1
            if (self._get_config("show_ram")):
                self.__do_ram(signal, index)
                index += 1
            if (self._get_config("show_net")):
                self.__do_net(signal, index)
                index += 1

            signal.set("size", index)

            self.__queue.put(signal)
            time.sleep(update / 1000)



def new_sensor(args): return apply(SysInfoReader, args)
